@@ -3,7 +3,7 @@ use rui::*;
 fn button_example() -> impl View {
     hstack((
         caption("button"),
-        button(text("press me"), |_| println!("pressed")),
+        button("press me", |_| println!("pressed")),
     ))
 }
 
@@ -11,8 +11,8 @@ fn slider_example() -> impl View {
     hstack((caption("slider"), state(|| 0.5, |s, _| hslider(s))))
 }
 
-fn caption(s: &str) -> impl View {
-    text(s).font_size(12).padding(Auto)
+fn caption(s: &'static str) -> impl View {
+    s.font_size(12).padding(Auto)
 }
 
 fn knob_example() -> impl View {
@@ -41,7 +41,7 @@ fn text_editor_example() -> impl View {
 
 fn main() {
     rui(vstack((
-        text("rui widget gallery"),
+        "rui widget gallery",
         button_example(),
         slider_example(),
         knob_example(),
