@@ -2,7 +2,7 @@ use rui::*;
 
 fn main() {
     rui(zstack((
-        text("This is a test."),
+        "This is a test.",
         circle().color(RED_HIGHLIGHT).padding(Auto),
     )));
 }
