@@ -19,7 +19,10 @@ pub trait Binding<S>: Clone + Copy + 'static {
     }
 }
 
-#[derive(Clone)]
+pub fn setter<S>(binding: impl Binding<S>) -> impl Fn(S, &mut Context) {
+    move |s, cx| binding.with_mut(cx, |v| *v = s)
+}
+
 pub struct Map<B, L, S, T> {
     binding: B,
     lens: L,
@@ -27,12 +30,24 @@ pub struct Map<B, L, S, T> {
     phantom_t: std::marker::PhantomData<T>,
 }
 
+impl<B, L, S, T> Clone for Map<B, L, S, T>
+where
+    B: Copy,
+    L: Copy,
+    S: 'static,
+    T: 'static,
+{
+    fn clone(&self) -> Self {
+        *self
+    }
+}
+
 impl<B, L, S, T> Copy for Map<B, L, S, T>
 where
     B: Copy,
     L: Copy,
-    S: Clone,
-    T: Clone,
+    S: 'static,
+    T: 'static,
 {
 }
 
@@ -40,8 +55,8 @@ impl<S, B, L, T> Map<B, L, S, T>
 where
     B: Binding<T>,
     L: Lens<T, S>,
-    S: Clone + 'static,
-    T: Clone + 'static,
+    S: 'static,
+    T: 'static,
 {
     pub fn new(binding: B, lens: L) -> Self {
         Self {
@@ -55,8 +70,8 @@ where
 
 pub fn bind<S, T>(binding: impl Binding<S>, lens: impl Lens<S, T>) -> impl Binding<T>
 where
-    S: Clone + 'static,
-    T: Clone + 'static,
+    S: 'static,
+    T: 'static,
 {
     Map::new(binding, lens)
 }
@@ -65,8 +80,8 @@ impl<S, B, L, T> Binding<S> for Map<B, L, S, T>
 where
     B: Binding<T>,
     L: Lens<T, S>,
-    S: Clone + 'static,
-    T: Clone + 'static,
+    S: 'static,
+    T: 'static,
 {
     fn get<'a>(&self, cx: &'a mut Context) -> &'a S {
         self.lens.focus(self.binding.get(cx))
@@ -97,7 +112,7 @@ mod tests {
 
     use super::*;
 
-    #[derive(Clone)]
+    #[derive(Clone, Default)]
     struct MyState {
         x: i32,
     }
@@ -112,12 +127,10 @@ mod tests {
     }
 
     #[test]
-    fn test_bind2() {
+    fn test_bind() {
         let mut cx = Context::new(None);
-        let id = ViewID::default();
-        cx.state_map
-            .entry(id)
-            .or_insert_with(|| Box::new(MyState { x: 0 }));
+        let id = ViewId::default();
+        cx.init_state(id, &MyState::default);
         let s = State::new(id);
 
         let b = bind(s, MyLens {});
