@@ -3,6 +3,8 @@ use euclid::*;
 use std::any::Any;
 use std::collections::HashMap;
 use std::ops;
+use tao::event::MouseButton;
+use std::any::TypeId;
 
 pub type LocalSpace = vger::defs::LocalSpace;
 pub type WorldSpace = vger::defs::WorldSpace;
@@ -12,6 +14,8 @@ pub type LocalSize = Size2D<f32, LocalSpace>;
 pub type LocalPoint = Point2D<f32, LocalSpace>;
 pub type WorldRect = Rect<f32, WorldSpace>;
 pub type WorldPoint = Point2D<f32, WorldSpace>;
+pub type LocalToWorld = Transform2D<f32, LocalSpace, WorldSpace>;
+pub type WorldToLocal = Transform2D<f32, WorldSpace, LocalSpace>;
 
 use tao::window::Window;
 
@@ -23,16 +27,23 @@ pub(crate) struct LayoutBox {
     pub offset: LocalOffset,
 }
 
-pub type StateMap = HashMap<ViewID, Box<dyn Any>>;
+pub(crate) struct StateHolder {
+    pub state: Box<dyn Any>,
+    pub dirty: bool,
+}
+
+pub(crate) type StateMap = HashMap<ViewId, StateHolder>;
+
+pub(crate) type EnvMap = HashMap<TypeId, Box<dyn Any>>;
 
 /// The Context stores all UI state. A user of the library
 /// shouldn't have to interact with it directly.
 pub struct Context {
     /// Layout information for all views.
-    pub(crate) layout: HashMap<ViewID, LayoutBox>,
+    pub(crate) layout: HashMap<ViewId, LayoutBox>,
 
     /// Which views each touch (or mouse pointer) is interacting with.
-    pub(crate) touches: [ViewID; 16],
+    pub(crate) touches: [ViewId; 16],
 
     /// Points at which touches (or click-drags) started.
     pub(crate) starts: [LocalPoint; 16],
@@ -40,11 +51,17 @@ pub struct Context {
     /// Previous touch/mouse positions.
     pub(crate) previous_position: [LocalPoint; 16],
 
+    /// Pressed mouse buton.
+    pub(crate) mouse_button: Option<MouseButton>,
+
+    /// Keyboard modifiers state.
+    pub(crate) key_mods: ModifiersState,
+
     /// The root view ID. This should be randomized for security reasons.
-    pub(crate) root_id: ViewID,
+    pub(crate) root_id: ViewId,
 
     /// The view that has the keybord focus.
-    pub(crate) focused_id: Option<ViewID>,
+    pub(crate) focused_id: Option<ViewId>,
 
     /// The tao window
     pub(crate) window: Option<Window>,
@@ -52,7 +69,7 @@ pub struct Context {
     /// The current title of the window
     pub(crate) window_title: String,
 
-    /// Attempt to not use interior mutability.
+    /// User state created by `state`.
     pub(crate) state_map: StateMap,
 
     /// Has the state changed?
@@ -60,22 +77,28 @@ pub struct Context {
 
     /// Are we currently setting the dirty bit?
     pub(crate) enable_dirty: bool,
+
+    /// Values indexed by type.
+    pub(crate) env: EnvMap,
 }
 
 impl Context {
     pub fn new(window: Option<Window>) -> Self {
         Self {
             layout: HashMap::new(),
-            touches: [ViewID::default(); 16],
+            touches: [ViewId::default(); 16],
             starts: [LocalPoint::zero(); 16],
             previous_position: [LocalPoint::zero(); 16],
-            root_id: ViewID { id: 1 },
+            mouse_button: None,
+            key_mods: ModifiersState::default(),
+            root_id: ViewId { id: 1 },
             focused_id: None,
             window,
             window_title: "rui".into(),
             state_map: HashMap::new(),
             dirty: false,
             enable_dirty: true,
+            env: HashMap::new()
         }
     }
 
@@ -85,11 +108,36 @@ impl Context {
         }
     }
 
+    pub(crate) fn clear_dirty(&mut self) {
+        self.dirty = false;
+        for holder in &mut self.state_map.values_mut() {
+            holder.dirty = false;
+        }
+    }
+
+    pub(crate) fn init_state<S: 'static, D: Fn() -> S + 'static>(&mut self, id: ViewId, func: &D) {
+        self.state_map.entry(id).or_insert_with(|| StateHolder {
+            state: Box::new((func)()),
+            dirty: false,
+        });
+    }
+
+    pub(crate) fn init_env<S: Clone + 'static, D: Fn() -> S + 'static>(&mut self, func: &D) -> S {
+        self.env.entry(TypeId::of::<S>()).or_insert_with(|| Box::new((func)())).downcast_ref::<S>().unwrap().clone()
+    }
+
+    pub(crate) fn set_env<S: Clone + 'static>(&mut self, value: &S) -> Option<S> {
+        let typeid = TypeId::of::<S>();
+        let old_value = self.env.get(&typeid).map(|b| b.downcast_ref::<S>().unwrap().clone());
+        self.env.insert(typeid, Box::new(value.clone()));
+        old_value
+    }
+
     pub fn get<S>(&self, id: State<S>) -> &S
     where
         S: 'static,
     {
-        self.state_map[&id.id].downcast_ref::<S>().unwrap()
+        self.state_map[&id.id].state.downcast_ref::<S>().unwrap()
     }
 
     pub fn get_mut<S>(&mut self, id: State<S>) -> &mut S
@@ -97,11 +145,10 @@ impl Context {
         S: 'static,
     {
         self.set_dirty();
-        self.state_map
-            .get_mut(&id.id)
-            .unwrap()
-            .downcast_mut::<S>()
-            .unwrap()
+
+        let mut holder = self.state_map.get_mut(&id.id).unwrap();
+        holder.dirty = true;
+        holder.state.downcast_mut::<S>().unwrap()
     }
 }
 
@@ -112,7 +159,7 @@ where
     type Output = S;
 
     fn index(&self, index: State<S>) -> &Self::Output {
-        self.state_map[&index.id].downcast_ref::<S>().unwrap()
+        self.get(index)
     }
 }
 
@@ -121,11 +168,6 @@ where
     S: 'static,
 {
     fn index_mut(&mut self, index: State<S>) -> &mut Self::Output {
-        self.set_dirty();
-        self.state_map
-            .get_mut(&index.id)
-            .unwrap()
-            .downcast_mut::<S>()
-            .unwrap()
+        self.get_mut(index)
     }
 }
