@@ -12,92 +12,26 @@ pub use event::*;
 mod binding;
 pub use binding::*;
 
-mod state;
-pub use state::*;
-
-mod text;
-pub use text::*;
-
-mod button;
-pub use button::*;
-
-mod stack;
-pub use stack::*;
-
 mod context;
 pub use context::*;
 
-mod padding;
-pub use padding::*;
-
-mod shapes;
-pub use shapes::*;
+mod views;
+pub use views::*;
 
 mod paint;
 pub use paint::*;
 
-mod gestures;
-pub use gestures::*;
-
-mod background;
-pub use background::*;
-
 mod modifiers;
 pub use modifiers::*;
 
-mod geom;
-pub use geom::*;
-
-mod offset;
-pub use offset::*;
-
-mod canvas;
-pub use canvas::*;
-
-mod slider;
-pub use slider::*;
-
-mod list;
-pub use list::*;
-
-mod size;
-pub use size::*;
-
-mod body;
-pub use body::*;
-
-mod knob;
-pub use knob::*;
-
-mod command;
-pub use command::*;
-
 mod colors;
 pub use colors::*;
 
-mod key;
-pub use key::*;
-
-mod text_editor;
-pub use text_editor::*;
-
-mod focus;
-pub use focus::*;
-
-mod cond;
-pub use cond::*;
-
-mod toggle;
-pub use toggle::*;
-
 mod align;
 pub use align::*;
 
-mod role;
-pub use role::*;
-
-mod window;
-pub use window::*;
+mod region;
+pub use region::*;
 
 use futures::executor::block_on;
 use std::collections::{HashMap, HashSet};
@@ -288,7 +222,7 @@ fn build_menubar(
 }
 
 /// Call this function to run your UI.
-pub fn rui(view: impl View + 'static) {
+pub fn rui(view: impl View) {
     let event_loop = EventLoop::new();
 
     let builder = WindowBuilder::new().with_title("rui");
@@ -324,10 +258,10 @@ pub fn rui(view: impl View + 'static) {
         .unwrap()
         .set_menu(Some(build_menubar(&commands, &mut command_map)));
 
-    let mut modifiers = ModifiersState::default();
-
     let mut access_nodes = vec![];
 
+    let mut dirty_region = Region::<WorldSpace>::EMPTY;
+
     event_loop.run(move |event, _, control_flow| {
         // ControlFlow::Poll continuously runs the event loop, even if the OS hasn't
         // dispatched any events. This is ideal for games and similar applications.
@@ -396,7 +330,7 @@ pub fn rui(view: impl View + 'static) {
                     // Clean up state.
                     let mut keep = vec![];
                     view.gc(cx.root_id, &mut cx, &mut keep);
-                    let keep_set = HashSet::<ViewID>::from_iter(keep);
+                    let keep_set = HashSet::<ViewId>::from_iter(keep);
                     cx.state_map.retain(|k, _| keep_set.contains(k));
 
                     // Get a new accesskit tree.
@@ -416,9 +350,17 @@ pub fn rui(view: impl View + 'static) {
                         // println!("access nodes unchanged");
                     }
 
+                    // Get dirty rectangles.
+                    view.dirty(
+                        cx.root_id,
+                        LocalToWorld::identity(),
+                        &mut cx,
+                        &mut dirty_region,
+                    );
+
                     cx.window.as_ref().unwrap().request_redraw();
 
-                    cx.dirty = false;
+                    cx.clear_dirty();
                 }
             }
             tao::event::Event::RedrawRequested(_) => {
@@ -455,6 +397,20 @@ pub fn rui(view: impl View + 'static) {
                 view.draw(cx.root_id, &mut cx, &mut vger);
                 cx.enable_dirty = true;
 
+                let paint = vger.color_paint(RED_HIGHLIGHT);
+                let xf = WorldToLocal::identity();
+                for rect in dirty_region.rects() {
+                    vger.stroke_rect(
+                        xf.transform_point(rect.min()),
+                        xf.transform_point(rect.max()),
+                        0.0,
+                        1.0,
+                        paint,
+                    );
+                }
+
+                dirty_region.clear();
+
                 let texture_view = frame
                     .texture
                     .create_view(&wgpu::TextureViewDescriptor::default());
@@ -477,11 +433,12 @@ pub fn rui(view: impl View + 'static) {
                 frame.present();
             }
             tao::event::Event::WindowEvent {
-                event: WindowEvent::MouseInput { state, .. },
+                event: WindowEvent::MouseInput { state, button, .. },
                 ..
             } => {
                 match state {
                     ElementState::Pressed => {
+                        cx.mouse_button = Some(button);
                         let event = Event {
                             kind: EventKind::TouchBegin { id: 0 },
                             position: mouse_position,
@@ -489,6 +446,7 @@ pub fn rui(view: impl View + 'static) {
                         view.process(&event, cx.root_id, &mut cx, &mut vger)
                     }
                     ElementState::Released => {
+                        cx.mouse_button = None;
                         let event = Event {
                             kind: EventKind::TouchEnd { id: 0 },
                             position: mouse_position,
@@ -520,7 +478,7 @@ pub fn rui(view: impl View + 'static) {
             } => {
                 if event.state == ElementState::Pressed {
                     let event = Event {
-                        kind: EventKind::Key(event.logical_key, modifiers),
+                        kind: EventKind::Key(event.logical_key),
                         position: mouse_position,
                     };
                     view.process(&event, cx.root_id, &mut cx, &mut vger)
@@ -530,8 +488,8 @@ pub fn rui(view: impl View + 'static) {
                 event: WindowEvent::ModifiersChanged(mods),
                 ..
             } => {
-                modifiers = mods;
-                // println!("modifiers changed: {:?}", modifiers);
+                // println!("modifiers changed: {:?}", mods);
+                cx.key_mods = mods;
             }
             tao::event::Event::MenuEvent { menu_id, .. } => {
                 //println!("menu event");
