@@ -1,15 +1,15 @@
 use std::collections::hash_map::DefaultHasher;
 use std::hash::{Hash, Hasher};
 
-/// `ViewID` is a unique identifier for a view. We're using a u64 and hashing
+/// `ViewId` is a unique identifier for a view. We're using a u64 and hashing
 /// under the assumption there won't be collsions. The underlying u64 is a function
 /// of the path down the view tree.
 #[derive(Copy, Clone, Default, Eq, PartialEq, Hash, Debug)]
-pub struct ViewID {
+pub struct ViewId {
     pub id: u64,
 }
 
-impl ViewID {
+impl ViewId {
     /// Computes the ID for a child using a hashable value. For views
     /// which don't have dynamic children (e.g. `vstack` etc.) the value
     /// will be the integer index of the child. Dynamic
